import time
import traceback
from base64 import b64decode, b64encode
from pathlib import Path

import httpx
import orjson
from httpx import TimeoutException
from nonebot.log import logger

from ..data_source import config

ocr_url = config.ocr_url
dir_path = Path(__file__).parent.parent
game_path = Path(__file__).parent
ocr_data_path = game_path / 'ocr_data.json'
upload_url = 'https://api.wows.shinoaki.com/api/wows/cache/image/ocr'
download_url = 'https://api.wows.shinoaki.com/api/wows/cache/image/ocr'

headers = {'Authorization': config.api_token}


async def pic2txt_byOCR(img_path, filename):
    try:
        if filename in ocr_filename_data:
            logger.success(f'filename匹配，跳过OCR:{filename}')
            return b64decode(ocr_filename_data[filename]).decode('utf-8')
        if config.ocr_offline:
            return ''
        start = time.time()
        params = {'url': img_path}
        async with httpx.AsyncClient() as client:
            resp = await client.post(f'{ocr_url}/OCR/', data=params, timeout=5, follow_redirects=True)
            end = time.time()
            result = orjson.loads(resp.content)
            if result['code'] == 200:
                logger.success(f"OCR结果：{result['data']['msg']},耗时{end-start:.4f}s\n图片url:{img_path}")
                return result['data']['msg']
    except TimeoutException:
        logger.error('ocr超时，请确认OCR服务是否在线')
        return ''
    except Exception:
        logger.error(traceback.format_exc())
        return ''


async def upload_OcrResult(result_text, filename):
    try:
        params = {
            'md5': filename,
            'text': b64encode(result_text.encode('utf-8')).decode('utf-8'),
        }
        async with httpx.AsyncClient(headers=headers, timeout=5) as client:
            resp = await client.post(upload_url, json=params)
            result = orjson.loads(resp.content)
            if result['code'] == 200:
                await downlod_OcrResult()
    except Exception:
        logger.error(traceback.format_exc())


async def downlod_OcrResult():
    try:
        global ocr_filename_data
        async with httpx.AsyncClient(headers=headers, timeout=10) as client:
            resp = await client.get(download_url)
            result = orjson.loads(resp.content)
            with open(ocr_data_path, 'w', encoding='UTF-8') as f:
                if result['code'] == 200 and result['data']:
                    f.write(orjson.dumps(result['data']).decode())
                    ocr_filename_data = result['data']
                else:
                    with open(ocr_data_path, 'rb') as f:  # noqa: PLW2901
                        ocr_filename_data = orjson.loads(f.read())
        return
    except Exception:
        logger.error('请检查token是否配置正确，如无问题请尝试重启，可能是网络波动或服务器原因')
        logger.error(traceback.format_exc())
        try:
            with open(ocr_data_path, 'rb') as f:
                ocr_filename_data = orjson.loads(f.read())
        except Exception:
            ocr_filename_data = None


async def get_Random_Ocr_Pic():
    try:
        async with httpx.AsyncClient() as client:
            resp = await client.post(f'{ocr_url}/ImageRandom/')
            img = b64decode(resp.text)
        return img
    except Exception:
        logger.error(traceback.format_exc())
        return 'OCR服务器出了点问题，请稍后再试哦'



import asyncio
import os
import platform
import re
import sys
import traceback
from collections import defaultdict, namedtuple

import httpx
from hikari_core import callback_hikari, init_hikari, set_hikari_config
from hikari_core.data_source import __version__
from hikari_core.game.help import check_version
from hikari_core.model import Hikari_Model
from hikari_core.moudle.wws_real_game import get_diff_ship
from nonebot import get_driver, on_command, on_fullmatch, on_message, require
from nonebot.adapters.onebot.v11 import (
    ActionFailed,
    Bot,
    GroupMessageEvent,
    Message,
    MessageEvent,
    MessageSegment,
    PrivateMessageEvent,
)
from nonebot.log import logger
from nonebot.params import CommandArg
from nonebot.permission import SUPERUSER
from nonebot_plugin_guild_patch import GuildMessageEvent

from .data_source import dir_path, nb2_file, template_path
from .game.ocr import (
    downlod_OcrResult,
    get_Random_Ocr_Pic,
    pic2txt_byOCR,
    upload_OcrResult,
)
from .game.pupu import get_pupu_msg
from .utils import DailyNumberLimiter, FreqLimiter, download, get_bot

scheduler = require('nonebot_plugin_apscheduler').scheduler

_max = 100
EXCEED_NOTICE = f'您今天已经冲过{_max}次了，请明早5点后再来！'
is_first_run = True
_nlmt = DailyNumberLimiter(_max)
_flmt = FreqLimiter(3)
__bot_version__ = '1.0.3'

bot_get_random_pic = on_fullmatch('wws 随机表情包', block=True, priority=5)
bot_update = on_fullmatch('wws 更新Hikari', priority=5, block=True, permission=SUPERUSER)
bot = on_command('wws', block=False, aliases={'WWS'}, priority=54)
bot_pupu = on_fullmatch('噗噗', block=False, priority=5)
bot_listen = on_message(priority=5, block=False)
ocr_listen = on_message(priority=6, block=False)
driver = get_driver()

_proxy = None
if driver.config.proxy_on:
    _proxy = driver.config.proxy

set_hikari_config(
    use_broswer=driver.config.htmlrender_browser,
    http2=driver.config.http2,
    proxy=_proxy,
    token=driver.config.api_token,
    game_path=str(dir_path / 'game'),
)

SlectState = namedtuple('SlectState', ['state', 'SlectIndex', 'SelectList'])
SecletProcess = defaultdict(lambda: SlectState(False, None, None))


@bot.handle()
async def main(bot: Bot, ev: MessageEvent, matchmsg: Message = CommandArg()):  # noqa: B008, PLR0915
    try:
        server_type = None
        if isinstance(ev, PrivateMessageEvent) and (driver.config.private or str(ev.user_id) in driver.config.superusers):  # 私聊事件,superusers默认不受影响
            server_type = 'QQ'
        elif isinstance(ev, GroupMessageEvent) and driver.config.group and ev.group_id not in driver.config.ban_group_list:  # 群聊事件
            server_type = 'QQ'
        elif isinstance(ev, GuildMessageEvent) and driver.config.channel:  # 频道事件
            if driver.config.all_channel or ev.channel_id in driver.config.channel_list:
                server_type = 'QQ_CHANNEL'
            else:
                return False
        else:
            return False
        qqid = ev.user_id
        group_id = None
        if ev.message_type == 'group':
            group_id = ev.group_id
        if not _nlmt.check(qqid):
            await bot.send(ev, EXCEED_NOTICE, at_sender=True)
            return False
        if not _flmt.check(qqid):
            await bot.send(ev, '您冲得太快了，请稍候再冲', at_sender=True)
            return False
        _flmt.start_cd(qqid)
        _nlmt.increase(qqid)
        superuser_command_list = ['重置监控']
        adminuser_command_list = ['添加监控', '删除监控']
        for each in superuser_command_list:
            if (each in str(ev.message) or each in matchmsg) and str(qqid) not in driver.config.superusers:
                await bot.send(ev, '该命令仅限超级管理员使用')
                return
        if str(qqid) not in driver.config.superusers:
            for each in adminuser_command_list:
                if (each in str(ev.message) or each in matchmsg) and qqid not in driver.config.admin_list:
                    await bot.send(ev, '请联系机器人搭建者添加权限')
                    return
        hikari = await init_hikari(platform=server_type, PlatformId=str(qqid), command_text=str(matchmsg), GroupId=group_id)
        if hikari.Status == 'success':
            if isinstance(hikari.Output.Data, bytes):
                await bot.send(ev, MessageSegment.image(hikari.Output.Data))
            elif isinstance(hikari.Output.Data, str):
                await bot.send(ev, hikari.Output.Data)
        elif hikari.Status == 'wait':
            await bot.send(ev, MessageSegment.image(hikari.Output.Data))
            hikari = await wait_to_select(hikari)
            if hikari.Status == 'error':
                await bot.send(ev, str(hikari.Output.Data))
                return
            hikari = await callback_hikari(hikari)
            if isinstance(hikari.Output.Data, bytes):
                await bot.send(ev, MessageSegment.image(hikari.Output.Data))
            elif isinstance(hikari.Output.Data, str):
                await bot.send(ev, str(hikari.Output.Data))
        else:
            await bot.send(ev, str(hikari.Output.Data))
    except ActionFailed:
        logger.warning(traceback.format_exc())
        try:
            await bot.send(ev, '发不出图片，可能被风控了QAQ')
            return True
        except Exception:
            pass
        return False
    except Exception:
        logger.error(traceback.format_exc())
        await bot.send(ev, '呜呜呜发生了错误，可能是网络问题，如果过段时间不能恢复请联系麻麻哦~')


@bot_listen.handle()
async def change_select_state(bot: Bot, ev: MessageEvent):
    try:
        # bot = get_bot()
        msg = str(ev.message)
        qqid = str(ev.user_id)
        if SecletProcess[qqid].state and str(msg).isdigit():
            if int(msg) <= len(SecletProcess[qqid].SelectList):
                SecletProcess[qqid] = SecletProcess[qqid]._replace(state=False)
                SecletProcess[qqid] = SecletProcess[qqid]._replace(SlectIndex=int(msg))
            else:
                await bot.send(ev, '请选择列表中的序号哦~')
        return
    except Exception:
        logger.error(traceback.format_exc())
        return


async def wait_to_select(hikari):
    SecletProcess[hikari.UserInfo.PlatformId] = SlectState(True, None, hikari.Input.Select_Data)
    a = 0
    while a < 40 and not SecletProcess[hikari.UserInfo.PlatformId].SlectIndex:
        a += 1
        await asyncio.sleep(0.5)
    if SecletProcess[hikari.UserInfo.PlatformId].SlectIndex:
        hikari.Input.Select_Index = SecletProcess[hikari.UserInfo.PlatformId].SlectIndex
        SecletProcess[hikari.UserInfo.PlatformId] = SlectState(False, None, None)
        return hikari
    else:
        SecletProcess[hikari.UserInfo.PlatformId] = SlectState(False, None, None)
        return hikari.error('已超时退出')


@ocr_listen.handle()
async def OCR_listen(bot: Bot, ev: MessageEvent):
    try:
        if not driver.config.ocr_on:
            return
        if not (str(ev.message).find('[CQ:image') + 1):  # 判断收到的信息是否为图片，不是就退出
            return
        tencent_url = ''
        for seg in ev.message:
            if seg.type == 'image':
                tencent_url = seg.data['url']
                filename = str(seg.data['file']).replace('.image', '')
        ocr_text = await pic2txt_byOCR(tencent_url, filename)
        if ocr_text:
            match = re.search(r'^(/?)wws(.*?)$', ocr_text)
            if match:
                searchtag = re.sub(r'^(/?)wws', '', ocr_text)  # 删除wws和/wws
                is_send = await main(bot, ev, searchtag)
                if is_send:
                    await upload_OcrResult(ocr_text, filename)
    except Exception:
        logger.error(traceback.format_exc())
        return


@bot_get_random_pic.handle()
async def send_random_ocr_image(bot: Bot, ev: MessageEvent):
    try:
        img = await get_Random_Ocr_Pic()
        if isinstance(img, bytes):
            await bot.send(ev, MessageSegment.image(img))
        elif isinstance(img, str):
            await bot.send(ev, str(img))
    except Exception:
        logger.error(traceback.format_exc())
        await bot.send(ev, '呜呜呜发生了错误，可能是网络问题，如果过段时间不能恢复请联系麻麻哦~')
        return


@bot_update.handle()
async def update_Hikari(ev: MessageEvent, bot: Bot):
    try:
        from nonebot_plugin_reboot import Reloader

        await bot.send(ev, '正在更新Hikari，完成后将自动重启，如果没有回复您已上线的消息，请登录服务器查看')
        if hasattr(driver.config, 'nb2_path'):
            # 并发fastgit会429，改为顺序请求
            for each in nb2_file:
                await download(each['url'], f"{driver.config.nb2_path}\\{each['name']}")
                await asyncio.sleep(0.5)

            # await asyncio.gather(
            #    *[download(each["url"], f"{driver.config.nb2_path}\{each['name']}") for each in nb2_file]
            # )
        logger.info(f'当前解释器路径{sys.executable}')
        os.system(f'{sys.executable} -m pip install hikari-bot -i https://pypi.tuna.tsinghua.edu.cn/simple --upgrade')
        os.system(f'{sys.executable} -m pip install hikari-core -i https://pypi.tuna.tsinghua.edu.cn/simple --upgrade')
        os.system(f'{sys.executable} -m pip install nonebot-plugin-gocqhttp -i https://pypi.tuna.tsinghua.edu.cn/simple --upgrade')
        Reloader.reload(delay=1)
    except RuntimeError:
        if str(platform.system()).lower() == 'linux':
            try:
                import multiprocessing

                for child in multiprocessing.active_children():
                    child.terminate()
                sys.stdout.flush()
                # not compatible with cmdline with '\n'
                os.execv(
                    os.readlink('/proc/self/exe'),
                    open('/proc/self/cmdline', 'rb').read().replace(b'\0', b'\n').decode().split('\n')[:-1],
                )
            except Exception:
                logger.error(traceback.format_exc())
                await bot.send(ev, '自动更新失败了QAQ，请登录服务器查看具体报错日志')
        else:
            logger.error(traceback.format_exc())
            await bot.send(ev, '不支持nb run启动的方式更新哦，请使用python bot.py 启动Hikari')
    except Exception:
        logger.error(traceback.format_exc())
        await bot.send(ev, '自动更新失败了QAQ，请登录服务器查看具体报错日志')


@driver.on_startup
async def startup():
    try:
        if driver.config.ocr_on:
            await downlod_OcrResult()
    except Exception:
        logger.error(traceback.format_exc())
        return


@driver.on_bot_connect
async def remind(bot: Bot):
    superid = driver.config.superusers
    await bot.get_login_info()
    for each in superid:
        await bot.send_private_msg(user_id=int(each), message=f'Hikari已上线，当前BOT版本{__bot_version__},内核版本{__version__}')


async def startup_download(url, name):
    async with httpx.AsyncClient() as client:
        resp = await client.get(url, timeout=20)
        with open(template_path / name, 'wb+') as file:
            file.write(resp.content)


async def job_chech_version():
    bot = get_bot()
    hikari = Hikari_Model()
    hikari = await check_version(hikari)
    superid = driver.config.superusers
    for each in superid:
        await bot.send_private_msg(user_id=int(each), message=hikari.Output.Data)


async def job_listen_battle():
    bot = get_bot()
    hikari = Hikari_Model()
    hikari = await get_diff_ship(hikari)
    if hikari.Status == 'success':
        for _each in hikari.Output.Data:
            await bot.send_group_msg(group_id=_each['group_id'], message=_each['msg'])


scheduler.add_job(job_chech_version, 'cron', hour=12)
scheduler.add_job(startup, 'cron', hour=4)
scheduler.add_job(downlod_OcrResult, 'interval', minutes=10)
scheduler.add_job(job_listen_battle, 'interval', minutes=driver.config.battle_listen_time)


@bot_pupu.handle()
async def send_pupu_msg(ev: MessageEvent, bot: Bot):
    try:
        if driver.config.pupu and isinstance(ev, GroupMessageEvent) and driver.config.group and ev.group_id not in driver.config.ban_group_list:
            msg = await get_pupu_msg()
            await bot.send(ev, msg)
    except ActionFailed:
        logger.warning(traceback.format_exc())
        try:
            await bot.send(ev, '噗噗寄了>_<可能被风控了QAQ')
        except Exception:
            pass
        return



import traceback
from asyncio.exceptions import TimeoutError
from pathlib import Path

import httpx
import orjson
from httpx import ConnectTimeout
from nonebot.log import logger

dir_path = Path(__file__).parent


async def get_pupu_msg():
    try:
        url = 'https://v1.hitokoto.cn'
        params = {}
        async with httpx.AsyncClient() as client:
            resp = await client.get(url, params=params, timeout=5)
            result = orjson.loads(resp.content)
            if resp.status_code == 200:
                return result['hitokoto']
            else:
                return '噗噗出问题了>_<'
    except (TimeoutError, ConnectTimeout):
        logger.warning(traceback.format_exc())
    except Exception:
        logger.error(traceback.format_exc())
        return '噗噗出问题了>_<'



from pathlib import Path

from nonebot import get_driver

dir_path = Path(__file__).parent
template_path = dir_path / 'template'
config = get_driver().config

nb2_file = [
    {'name': 'bot.py', 'url': 'https://raw.fastgit.org/benx1n/HikariBot/master/bot.py'},
    {'name': '.env', 'url': 'https://raw.fastgit.org/benx1n/HikariBot/master/.env'},
    {'name': 'pyproject.toml', 'url': 'https://raw.fastgit.org/benx1n/HikariBot/master/pyproject.toml'},
]
